// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.BorderFactory; 
import javax.swing.border.*;
import javax.swing.Box.Filler;
import java.util.*;


class HistoryWindow
	// History-Window for Visual Marriage Applet.
	// logs all action as a history and tells it to the user.

extends 

	JFrame

{	

	private JPanel history_panel;
		// the panel conatining all the history messages.
		
	private JScrollPane scroll_pane;
		// the scroll pane to scroll the messages.

  	public HistoryWindow()
	// initialize history frame window
	{
		
		// create GUI.
		buildGUI (getContentPane());
				
  	}
	
    private void buildGUI(Container container) 
	// Build main parts of the GUI.
	{	
		// set options.
		int width = 360;
		int height = 520;
		setBackground (Color.black);
		container.setBackground (Color.black); 
		setTitle ("VISUAL MARRIAGE - History");
		setResizable (false);
		
		// create history_panel to hold all history messages.
		history_panel = new JPanel();
		history_panel.setBackground(Color.black);
		BoxLayout layout = new BoxLayout (history_panel, BoxLayout.Y_AXIS);
		history_panel.setLayout (layout);		
		history_panel.setAlignmentY(history_panel.TOP_ALIGNMENT);

		// create scroll_pane to scroll the history_panel.
		scroll_pane = new JScrollPane();
		scroll_pane.setViewportView (history_panel);
		scroll_pane.getViewport().setBackground(Color.black);
		// scroll_pane.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED, Color.gray, Color.darkGray));
		container.add(scroll_pane);
		scroll_pane.getVerticalScrollBar().setUnitIncrement(20);
		
		// Add a vertical space at the beginning of the history_panel.
		history_panel.add(getAVSpace(10));
		
		// lay out everything & set size.
		setSize(new Dimension(width, height));					

  	}
	
	public void clear() {
	
		// remove all messages.
		history_panel.removeAll();

		// Add a vertical space at the beginning of the history_panel.
		history_panel.add(getAVSpace(10));
		
		// repaint everything.
		history_panel.validate();
		history_panel.repaint();
		scroll_pane.validate();
		scroll_pane.repaint();
	}
	
	public void addProposalMessage(Person man, Person woman) 
	// add message to 'history_panel' thath 'man' asks 'woman' to marry.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(man.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" asks "));
		message_panel.add(woman.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" to marry."));
		
		// repaint everything.
		history_panel.validate();
		history_panel.repaint();
		scroll_pane.validate();
		scroll_pane.repaint();		
	}
	
	public void addFreeWomanMessage(Person woman) 
	// add message to 'history_panel' that 'woman' is free and accepts.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(woman.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" is not married yet and accepts."));
				
		// paint everything.		
		history_panel.validate();
		history_panel.repaint();
		scroll_pane.validate();
		scroll_pane.repaint();
		
	}

	public void addMarriedWomanMessage(Person woman, Person partner) 
	// add message to 'history_panel' that 'woman' is already married.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(woman.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" is already married with "));
		message_panel.add(partner.createSymbolWithNameComponent());
		message_panel.add(getALabel("."));
						
		// paint everything.	
		history_panel.validate();
		history_panel.repaint();	
		scroll_pane.validate();
		scroll_pane.repaint();
		
	}
	
	public void addDivorceMessage(Person man, Person woman) 
	// add message to 'history_panel' that 'woman' is already married.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(woman.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" and "));
		message_panel.add(man.createSymbolWithNameComponent());
		message_panel.add(getALabel(" get divorced."));
						
		// paint everything.	
		history_panel.validate();
		history_panel.repaint();	
		scroll_pane.validate();
		scroll_pane.repaint();
		
	}

	public void addLoveMessage(String prefix, Person man1, Person man2) 
	// add message to 'history_panel' that 'man1' is loved more than 'man2'.
	// Print 'prefix' text in front of the message.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(getALabel(prefix + " loves "));
		message_panel.add(man1.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" more than "));
		message_panel.add(man2.createSymbolWithNameComponent());
		message_panel.add(getALabel("."));
						
		// paint everything.	
		history_panel.validate();
		history_panel.repaint();	
		scroll_pane.validate();
		scroll_pane.repaint();
		
	}
	
	public void addMarriageMessage(Person woman, Person man) 
	// add message to 'history_panel' that 'woman' marries 'man'.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(woman.createSymbolWithNameComponent ());
		message_panel.add(getALabel(" marries "));
		message_panel.add(man.createSymbolWithNameComponent());
		message_panel.add(getALabel("."));
		
		// add a vertical space to 'history_panel'.
		history_panel.add(getAVSpace(5));		
		history_panel.add(getASeparator());
		history_panel.add(getAVSpace(5));
		
		// paint everything.	
		history_panel.validate();
		history_panel.repaint();	
		scroll_pane.validate();

		// scroll to message.
		scrollToMessage(message_panel);
								
	}
	
	public void addDenialMessage(Person man) 
	// add message to 'history_panel' that 'man' has to search on.
	{

		// Create panel for the message.
		JPanel message_panel = getAMessagePanel();
			
		// add message to message panel.
		message_panel.add(getAHSpace(10));
		message_panel.add(man.createSymbolWithNameComponent());
		message_panel.add(getALabel(" has to search on ..."));
		
		// add a vertical space to 'history_panel'.
		history_panel.add(getAVSpace(5));		
		history_panel.add(getASeparator());
		history_panel.add(getAVSpace(5));
		
		// paint everything.	
		history_panel.validate();
		history_panel.repaint();	
		scroll_pane.validate();

		// scroll to message.
		scrollToMessage(message_panel);
				
	}
	
	private void scrollToMessage(JPanel message) {
		scroll_pane.getVerticalScrollBar().setValue(message.getY()-300);	
		scroll_pane.repaint(); // make new scroll position clean visible 
		
	}


	private JPanel getAMessagePanel()
	// create a one line panel for a message.
	{
		// create message_panel.
		JPanel message_panel = new JPanel();
		message_panel.setBackground(Color.black);
		BoxLayout layout = new BoxLayout (message_panel, BoxLayout.X_AXIS);
		message_panel.setLayout (layout);				
		message_panel.setAlignmentX(message_panel.LEFT_ALIGNMENT);		
		message_panel.setAlignmentY(message_panel.TOP_ALIGNMENT);
		
		// add message panel to history_panel.
		history_panel.add(message_panel);
	
		// set message panel size.	
		message_panel.setMinimumSize(new Dimension(200, 16));
		message_panel.setMaximumSize(new Dimension(1000, 16));
		message_panel.setSize(new Dimension(280, 16));
		
		return message_panel;		
	}

	private JPanel getASeparator() 
	// create and return a white separator line panel.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.white);
		separator.setMinimumSize(new Dimension(1, 1));
		separator.setMaximumSize(new Dimension(1000, 1));
		separator.setAlignmentY(separator.TOP_ALIGNMENT);	
		return separator;
	}

	private JPanel getAVSpace(int height) 
	// create and return an empty JPanel to enforce the current line to be least 'height' high in flow layout.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.black);
		separator.setMinimumSize(new Dimension(0, height));
		separator.setMaximumSize(new Dimension(200, height));
		separator.setAlignmentY(separator.TOP_ALIGNMENT);	
		return separator;
	}
	
	private JPanel getAHSpace(int width) 
	// create and return an empty JPanel to enforce an indent in flow layout.
	// 'width' specifies the indent width in pixels.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.black);
		separator.setMinimumSize(new Dimension(width, 0));
		separator.setPreferredSize(new Dimension(width, 0));
		separator.setMaximumSize(new Dimension(width, 0));
		separator.setSize(new Dimension(width, 0));
		separator.setAlignmentX(separator.LEFT_ALIGNMENT);	
		return separator;
	}	
		
	private JLabel getALabel(String text)
	// create and return a label with standard formatting.
	{
		JLabel label = new JLabel(text);
		label.setForeground(Color.white);
		return label;	
	}

}
