// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.BorderFactory; 
import javax.swing.border.*;
import javax.swing.Box.Filler;
import java.util.*;


class OptionsWindow
	// Options-Window for Visual Marriage Applet.
	// contains the GUI to control some options.

extends 

	JFrame

implements 

	ActionListener,
	ChangeListener,
	ItemListener,
	KeyListener,
	Icon
	
{	

	private MainWindow main_window;
		// the main window that should be controlled by this control window.

	private JPanel main_panel, speed_panel, tables_panel, world_panel;
		// panels to group the options in some categories
	
	private JComboBox person_count_combo;
		// Combo box to choose the number of men / women.
		
	private JButton count_change_button;
		// Button to change the number of persons to the entered or choosen number of persons.

	private JSlider normal_speed_slider, slow_speed_slider;
		// Sliders to choose speeds.	
			
	private JCheckBox show_married_names_chkbox, show_acting_names_chkbox;
		// check box to control the visualisation in the world panel.	
	
	private JRadioButton name_only_option, symbol_only_option, symbol_and_name_option;
		// radio buttons to choose how to visualize preference lists in the tables.
		
	private ButtonGroup tables_option_group;
		// an option button group to group the table options radio buttons together.
		// (only one of them can be choosen at any time)
		
	public OptionsWindow(MainWindow parent)
	// initialize options frame window
	// with its parent MainWindow it should control.
	{
		
		// remember reference to parent main window.
		main_window = parent;
		
		// create GUI.
		buildGUI (getContentPane());
		
		// set default options.
		person_count_combo.setSelectedItem("9");		
		show_acting_names_chkbox.setSelected(true);
		main_window.setNamesVisibleForActingPersons(true);
		show_married_names_chkbox.setSelected(true);
		main_window.setNamesVisibleForMarriedPersons(true);		

  	}
		
	private JPanel getAFlowBreak() 
	// create and return an empty JPanel to enforce a new line in flow layout.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.black);
		separator.setPreferredSize(new Dimension(500, 0));		
		return separator;
	}
	
	private JPanel getAVSpace(int height) 
	// create and return an empty JPanel to enforce a new line in flow layout.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.black);
		separator.setPreferredSize(new Dimension(0, height));
		return separator;
	}
	
	private JPanel getAFlowIndent(int width) 
	// create and return an empty JPanel to enforce an indent in flow layout.
	// 'width' specifies the indent width in pixels.
	{
		JPanel separator = new JPanel();
		separator.setBackground(Color.black);
		separator.setPreferredSize(new Dimension(width, 10));		
		return separator;
	}	

	private JPanel getAPanel(Border border)
	// create and return a panel for a group of controls.
	{
		JPanel panel = new JPanel();
		panel.setBackground(Color.black); 
		FlowLayout layout = new FlowLayout (FlowLayout.LEFT);
		layout.setVgap(0);
		panel.setLayout (layout);
		panel.setBorder(border);
		return panel;	
	}


	private JLabel some_label;

	private JLabel getALabel(String text)
	// create and return a label with standard formatting.
	{
		JLabel label = new JLabel(text);
		label.setForeground(Color.white);		
		label.setPreferredSize(new Dimension(label.getPreferredSize().width, 22));
		label.setMinimumSize(new Dimension(label.getMinimumSize().width, 22));	
		if (some_label == null) {some_label = label;}
		return label;	
	}
	
	private JRadioButton getARadioButton(String text) 
	// create a radio button with the given text and standard formatting.
	{
		JRadioButton radio = new JRadioButton(text);
		radio.setBackground (Color.black);
		radio.setForeground (Color.white);
		radio.setIcon(this);  // drawing a nicer icon, with centered circles.
		radio.setPreferredSize(new Dimension(radio.getPreferredSize().width, 30));
		radio.setMinimumSize(new Dimension(radio.getMinimumSize().width, 30));
		return radio;		
	}

	private JCheckBox getACheckBox(String text) 
	// create a check box with the given text and standard formating.
	{
		JCheckBox chk = new JCheckBox(text);
		chk.setBackground(Color.black);
		chk.setForeground (Color.white);
		chk.setIcon (this);   // necessary, because selected icon has black color, not visible on black background :-(	
		chk.addItemListener(this);
		chk.setSelected(true);	
		chk.setPreferredSize(new Dimension(chk.getPreferredSize().width, 30));
		chk.setMinimumSize(new Dimension(chk.getMinimumSize().width, 30));	
		return chk;		
	}

	private JLabel getASliderLabel(int value)
	// create and return a label with standard formatting.
	{
		JLabel label = new JLabel(String.valueOf(value));
		label.setForeground(new Color(102, 102, 153));
		return label;	
	}
	
	private JSlider getASlider(int value, int min, int max, int tickspace) 
	// create a slider with given value and min/max values in standard formating.
	{
		JSlider slider = new JSlider(JSlider.HORIZONTAL, min, max, value);		
		slider.setBackground (Color.black);
		slider.setForeground (Color.gray);
		slider.addChangeListener(this);
		slider.setMajorTickSpacing(tickspace * 10);
		slider.setMinorTickSpacing(tickspace);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		
		Hashtable labelTable = new Hashtable();
		labelTable.put( new Integer(min), getASliderLabel(min) );
		labelTable.put( new Integer((min + (max-min)/4)), getASliderLabel((min + (max-min)/4)) );
		labelTable.put( new Integer(((max+min)/2)), getASliderLabel(((max+min)/2)) );
		labelTable.put( new Integer((min + 3*(max-min)/4)), getASliderLabel((min + 3*(max-min)/4)) );
		labelTable.put( new Integer(max), getASliderLabel(max) );
		slider.setLabelTable( labelTable );
		
		slider.setPreferredSize(new Dimension(slider.getPreferredSize().width, 50));
		slider.setMinimumSize(new Dimension(slider.getMinimumSize().width, 50));	
				
		return slider;		
	}


    private void buildGUI(Container container) 
	// Build main parts of the GUI.
	{	
		// set options.
		setBackground (Color.black);
		container.setBackground (Color.black); 
		container.setLayout (new FlowLayout (FlowLayout.CENTER));
		setTitle ("VISUAL MARRIAGE - Options");
		setResizable (false);
		
		// create nice border for panels.
		Border panel_border = BorderFactory.createEtchedBorder(EtchedBorder.RAISED, Color.gray, Color.darkGray);

		// create paneltypingArea.addKeyListener(this);
		main_panel = getAPanel(panel_border);
		container.add(main_panel);
		speed_panel = getAPanel(panel_border);
		container.add(speed_panel);
		world_panel = getAPanel(panel_border);
		container.add(world_panel);
		tables_panel = getAPanel(panel_border);
		container.add(tables_panel);
		
		// create controls for main_panel.
		Color button_color = new Color(64, 64, 192);
		main_panel.add(getALabel("Number of Women / Men: "));		
		main_panel.add(getAFlowBreak());
		main_panel.add(getAVSpace(50));
		main_panel.add(getAFlowIndent(30));		
		person_count_combo = new JComboBox();	
		person_count_combo.setBackground(button_color);
		person_count_combo.setForeground(Color.white);
		int count = 1;
		for (int i=1; i<=30; i++) {
			person_count_combo.addItem(String.valueOf(count));
			if (count < 20) {
				count = count + 1;
			}
			else {
				count = count + 5;			
			}
		}
		person_count_combo.setEditable(true);		
		person_count_combo.setPreferredSize(new Dimension(100, 24));
		person_count_combo.getEditor().getEditorComponent().addKeyListener(this);		
		main_panel.add(person_count_combo);				
		count_change_button = new JButton("change");
		count_change_button.setBackground(button_color);
		count_change_button.setForeground(Color.white);
		count_change_button.setMargin(new Insets(0,0,0,0));
		count_change_button.setPreferredSize(new Dimension(75, 24));
		count_change_button.setActionCommand("change");
		count_change_button.addActionListener(this);
		main_panel.add(count_change_button);
		
		// create controls for speed_panel.
		speed_panel.add(getALabel("Normal Speed: "));
		speed_panel.add(getAFlowBreak());
		speed_panel.add(getAFlowIndent(30));
		normal_speed_slider = getASlider(500, 0, 2000, 50);
		speed_panel.add(normal_speed_slider);		
		speed_panel.add(getAFlowBreak());
		speed_panel.add(getAFlowIndent(30));
		speed_panel.add(getAFlowBreak());
		speed_panel.add(getALabel("Slow Motion Speed: "));
		speed_panel.add(getAFlowBreak());
		speed_panel.add(getAFlowIndent(30));
		slow_speed_slider = getASlider(100, 0, 800, 20);
		speed_panel.add(slow_speed_slider);		
		
		// create controls for world_panel.
		world_panel.add(getALabel("Show Names in Animation ..."));
		world_panel.add(getAFlowBreak());
		world_panel.add(getAFlowIndent(30));
		show_married_names_chkbox = getACheckBox("for married persons");		
		world_panel.add(show_married_names_chkbox);		
		world_panel.add(getAFlowBreak());
		world_panel.add(getAFlowIndent(30));
		show_acting_names_chkbox = getACheckBox("for persons in action");
		world_panel.add(show_acting_names_chkbox);				
		
		// create controls for tables_panel.
		tables_panel.add(getALabel("Show Preferences in Tables as ..."));
		tables_panel.add(getAFlowBreak());
		tables_panel.add(getAFlowIndent(30));
		symbol_only_option = getARadioButton("Symbols only");
		tables_panel.add(symbol_only_option);
		tables_panel.add(getAFlowBreak());
		tables_panel.add(getAFlowIndent(30));
		name_only_option = getARadioButton("Names only");
		tables_panel.add(name_only_option);
		tables_panel.add(getAFlowBreak());
		tables_panel.add(getAFlowIndent(30));
		symbol_and_name_option = getARadioButton("Symbols and Names");
		tables_panel.add(symbol_and_name_option);
		tables_panel.add(getAFlowBreak());		
		tables_option_group = new ButtonGroup();		
		tables_option_group.add (symbol_only_option);
		tables_option_group.add (name_only_option);
		tables_option_group.add (symbol_and_name_option);
		
		// add actions to table options.
		symbol_only_option.addActionListener(
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					main_window.showSymbolsOnlyInTables();
				}
			}
		);		
		name_only_option.addActionListener(
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					main_window.showNamesOnlyInTables();
				}
			}
		);		
		symbol_and_name_option.addActionListener(
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					main_window.showSymbolsAndNamesInTables();
				}
			}
		);		
		symbol_only_option.setSelected(true);
				
		// lay out everything & set size.
		int width = 300;
		int height = 530;
		main_panel.setPreferredSize(new Dimension (width - 20, 75));
		speed_panel.setPreferredSize(new Dimension (width - 20, 170));
		world_panel.setPreferredSize(new Dimension (width - 20, 90));
		tables_panel.setPreferredSize(new Dimension (width - 20, 120));
		setSize(new Dimension(width, height));				

  	}
	
	public void changePersonCount() 
	// Change the number of persons to the one entered in the combo box.
	// If the entered number is not valid a error message dialog will pop up.
	{
		String scount = (String)(person_count_combo.getSelectedItem());
		int count = -1;
		try {
			count = Integer.parseInt(scount);				
		} 
		catch (Exception ex) {      		
			count = -1;				
		}
		if (count > 0 && count <= 500) {
			main_window.createPersons(count);
		} 
		else {
			JOptionPane.showMessageDialog(null, "Chosse a number from 1 to 500!", "Invalid Number Of Persons", JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	public void actionPerformed(ActionEvent e) 
	// Change the number of persons (performed when pressing the change button)
	{		
	    if (e.getActionCommand().equals("change")) {
			changePersonCount();			
	    }
   	}
	
	public void stateChanged(ChangeEvent e)
	// change the speed of persons (performed when a speed slider is moved)
	{
		double nspeed = ((double)(normal_speed_slider.getValue()));
		double sspeed = ((double)(slow_speed_slider.getValue()));		
		Person.speed = nspeed / 500.0;
		if (Person.speed <= 0.01) {
			Person.speed = 0.01;
		}
		Person.slow_speed = sspeed / 500.0;
		if (Person.slow_speed <= 0.01) {
			Person.slow_speed = 0.01;
		}
	}	
	
	public void	itemStateChanged(ItemEvent e) {
		JCheckBox chkbox = (JCheckBox)(e.getItemSelectable());
		if (chkbox == show_acting_names_chkbox) {
			main_window.setNamesVisibleForActingPersons(chkbox.isSelected());
		}
		if (chkbox == show_married_names_chkbox) {
			main_window.setNamesVisibleForMarriedPersons(chkbox.isSelected());
		}
	}

	public void keyTyped(KeyEvent e) 
	// Part of KeyListener, not implemented here.
	{
    }

	public void keyPressed(KeyEvent e) 
	// Part of KeyListener, not implemented here.
	{
	}
	
	public void keyReleased(KeyEvent e) 
	// Performed after a key in the combo box text field was pressed.
	// When the user released the ENTER key the number of persons should be changed.
	{
		if (e.getKeyCode() == e.VK_ENTER) {
			changePersonCount();			
		}
    }
	
	public void paintIcon(Component c, Graphics g, int x, int y)
	// Paint icon either for checkbox or radio button
	// workaround for Java Checkbox Bug (not taking right foreground color for selected check box).
	// and nicer icon for java radio button (centered circles instead of unnice uncentered).
	{
		Graphics2D g2d = (Graphics2D)g;		
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		int icon_size = 10;		
		
		// paint icon for a check box.
		if (c.getClass().getName()== "javax.swing.JCheckBox") {			
			JCheckBox chk = (JCheckBox)c;	
			g.setColor(Color.white);		
			g.drawRect(x, y, icon_size, icon_size);		
			if (chk.isSelected()) {			
				g.drawLine(x + 3, y + 3, x + icon_size - 3, y + icon_size - 3);
				g.drawLine(x + 3, y + icon_size - 3, x + icon_size - 3, y + 3);
			}
		}

		// paint icon for a radio button.
		if (c.getClass().getName()== "javax.swing.JRadioButton") {
			JRadioButton radio = (JRadioButton)c;	
			g.setColor(Color.white);		
			g.fillOval(x, y, icon_size, icon_size);
			g.setColor(Color.black);		
			g.fillOval(x + 1, y + 1, icon_size - 2, icon_size - 2);
			g.setColor(Color.white);
			if (radio.isSelected()) {						
				g.fillOval(x + 3, y + 3, icon_size - 6, icon_size - 6);
			}
		}
				
	}
	
	
	public int getIconWidth()
	// get the width of the check box icon (see paintIcon-workaround for check boxes).
	{
		return 14;
	}
	
	
	public int getIconHeight()
	// get the height of the check box icon (see paintIcon-workaround for check boxes).
	{
		return 10;
	}
	
}
