// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

public class PersonTableModel
	// Model of Table with persons.

extends AbstractTableModel

{
	
	public PersonTableModel() {
	
		// create and intialize column names.
		column_names = new Vector();
		column_names.add ("Symbol");
		column_names.add ("Name");
		column_names.add ("Preferences      <high           low>");
		
		// create and initialize some row_data (as empty).
		row_data = new Vector();		
	}
	
	private Vector column_names;
	// a Vector containing the captions for each column.
	
	private Vector row_data;
	// a Vector containing the row datas as Vectors again.
	
	public void add(Person p)
	// adds a Person `p' to `row_data'.
	{		
		// add p.
		row_data.add (p);
		
		// notify views.
		fireTableRowsInserted (row_data.size (), row_data.size ());
	}
	
	public Person elementAt(int row)
	// return person in `row'.
	{
		return (Person) row_data.elementAt (row);
	}
	
	
	public void removeAllElements() 
	// remove all rows.
	{
		// remove everything.
		row_data.removeAllElements();
		
		// notify views.
		fireTableDataChanged ();
	}
	
	public void extend_preferences (Person p)
	// extends preferences of all persons with `p'.
	{

		// extend each row with `p'.
		Enumeration enum_rows = row_data.elements();
		while (enum_rows.hasMoreElements()) {
			Person cur = (Person) enum_rows.nextElement ();
			cur.extend_preferences (p);
		}			
		
		// notify views.
		fireTableDataChanged ();
	}
	
	public Enumeration elements () {
		return row_data.elements();
	}
	
	public String getColumnName(int col) { 
        	return column_names.elementAt (col).toString(); 
	}
	
	public Object getColumnIdentifier (int col) {
		return column_names.elementAt (col);
	}

	public int getRowCount() { 
		return row_data.size (); 
	}

	public int getColumnCount() { 
		return column_names.size (); 
	}
	
	public Object getValueAt(int row, int col) { 
	
		Person p = (Person) row_data.elementAt(row);
	
		if (col == 0) return p;
		if (col == 1) return p.getName();
		if (col == 2) return p.getPreferences();
		
		return null;		

    	}
    
    	public boolean isCellEditable(int row, int col) {
		if (col == 1) return true;
		return false; 
	}
	
	public Class getColumnClass(int c) {
    	return getValueAt(0, c).getClass();
    }
	
	public void setValueAt(Object value, int row, int col) {

		Person p = (Person) row_data.elementAt(row);
	
		if (col == 1) p.setName ((String) value);			
		
		fireTableDataChanged ();
    }

}
